@@ -1,5 +1,6 @@
 from pandas import DataFrame
 
+import pickle
 import pkg_resources
 
 from .metadata import __copyright__, __license__, __version__
@@ -9,15 +10,15 @@ class JavaBackend(object):
 	def __init__(self):
 		pass
 
+	def dumps(self, arguments):
+		return pickle.dumps(arguments, protocol = 2)
+
+	def loads(self, results):
+		return pickle.loads(results)
+
 	def newObject(self, className, *args):
 		raise NotImplementedError()
 
-	def dict2map(self, pyDict):
-		raise NotImplementedError()
-
-	def map2dict(self, javaMap):
-		raise NotImplementedError()
-
 	def staticInvoke(self, className, methodName, *args):
 		raise NotImplementedError()
 
@@ -77,19 +78,16 @@ class Evaluator(JavaObject):
 		return self.outputFields
 
 	def evaluate(self, arguments):
-		javaArguments = self.backend.dict2map(arguments)
-		javaArguments = self.backend.staticInvoke("org.jpmml.evaluator.EvaluatorUtil", "encodeKeys", javaArguments)
-		javaResults = self.javaEvaluator.evaluate(javaArguments)
-		javaResults = self.backend.staticInvoke("org.jpmml.evaluator.EvaluatorUtil", "decodeAll", javaResults)
-		results = self.backend.map2dict(javaResults)
+		arguments = self.backend.dumps(arguments)
+		results = self.backend.staticInvoke("org.jpmml.evaluator.python.PythonUtil", "evaluate", self.javaEvaluator, arguments)
+		results = self.backend.loads(results)
 		return results
 
 	def evaluateAll(self, arguments_df):
 		argument_records = arguments_df.to_dict(orient = "records")
-		result_records = []
-		for argument_record in argument_records:
-			result_record = self.evaluate(argument_record)
-			result_records.append(result_record)
+		argument_records = self.backend.dumps(argument_records)
+		result_records = self.backend.staticInvoke("org.jpmml.evaluator.python.PythonUtil", "evaluateAll", self.javaEvaluator, argument_records)
+		result_records = self.backend.loads(result_records)
 		return DataFrame.from_records(result_records)
 
 class BaseModelEvaluatorBuilder(JavaObject):
@@ -153,13 +151,16 @@ def make_evaluator(backend, path, locatable = False, reporting = False):
 		evaluatorBuilder.setReportingValueFactoryFactory()
 	return evaluatorBuilder.build()
 
+def _package_data_jars(package_data_dir):
+	jars = []
+	resources = pkg_resources.resource_listdir(package_data_dir, "")
+	for resource in resources:
+		if resource.endswith(".jar"):
+			jars.append(pkg_resources.resource_filename(package_data_dir, resource))
+	return jars
+
 def _classpath(user_classpath):
 	return _package_classpath() + user_classpath
 
 def _package_classpath():
-	jars = []
-	resources = pkg_resources.resource_listdir("jpmml_evaluator.resources", "")
-	for resource in resources:
-		if resource.endswith(".jar"):
-			jars.append(pkg_resources.resource_filename("jpmml_evaluator.resources", resource))
-	return jars
+	return _package_data_jars("jpmml_evaluator.resources") + _package_data_jars("jpmml_evaluator.dependencies")
\ No newline at end of file
